"""
 AnswerSearchkit strictly defines the relationship between different API
 interfaces and the corresponding table style sheet, which we define as a
 mapping, and the mapping is unique for each API interface.
"""
from .models import Mapping
from typing import Dict






mapping:Dict[str,bool] = {

    "icodef":Mapping().setting_mapping("icodef",["Number","Question","Option","Content","API InterFace"]),

    # Add the new API-Table mapping rules here

}



class CSVkitInputTypeError(Exception):
    def __init__(self,_errorInput):
        Exception.__init__(self)
        self._errorMessage = _errorInput
    def __str__(self):
        return "输入数据 %s 的类型与函数要求所冲突"%self._errorMessage







class CSVkitImportError(Exception):
    def __init__(self,_packageName):
        Exception.__init__(self)
        self._packageName = _packageName
    def __str__(self):
        return "CSVkit所依赖的包 %s 未安装 \n" \
               "可以使用豆瓣源进行快速安装:  pip3 install -i https://pypi.doubanio.com/simple/ %s"%(self._packageName,self._packageName)


from .models import CSVobjects,WriteCSVobject
from datetime import datetime
import csv,os

class Operate():

    @staticmethod
    def return_file_message(PATH:str) -> dict:
        this_file_message_objects = os.stat(PATH)
        st_size = this_file_message_objects.st_size # File size
        st_mtime = str(datetime.fromtimestamp(this_file_message_objects.st_mtime))  # Modify time
        st_ctime = str(datetime.fromtimestamp(this_file_message_objects.st_ctime))  # Create time
        file_message = {"size": st_size, "mtime": st_mtime, "ctime": st_ctime}
        return file_message


    def read(self,PATH:str,Reading_method:str) -> CSVobjects:
        with open(PATH,Reading_method) as this_file_read:
            csv_reader = csv.reader(this_file_read)
            _content_List_:[[str]] = []

            for i in csv_reader:
                _content_List_.append(i)

            return CSVobjects(_content=_content_List_,_fileMessage=self.return_file_message(PATH))


    def write(self,PATH:str,header:[],rows:[[]],Writing_method:str) -> WriteCSVobject:
        with open(PATH, Writing_method) as f:
            f_csv = csv.writer(f)
            f_csv.writerow(header)
            f_csv.writerows(rows)
            # csv op is end
        return WriteCSVobject(_content=self.read(PATH, "r")._content,_fileMessage=self.return_file_message(PATH),_path=PATH)










class ApiTypeError(Exception):
    def __init__(self,_error_str):
        self._error_str = _error_str
    def __str__(self):
        return "API接口 %s 暂时不被支持"%self._error_str

class MatchError(Exception):
    def __init__(self,_url):
        self._url = _url
    def __str__(self):
        return "该网址 %s 暂无可用匹配规则"%self._url

class ThirdPackageImportError(Exception):
    def __init__(self,name):
        self.name = name
    def __str__(self):
        return "AnswerSearchkit所依赖的第三方库 %s 导入失败，请查明错误原因。\n若未下载，可以使用" \
               "豆瓣源进行快速安装:  pip3 install -i https://pypi.doubanio.com/simple/ %s"%(self.name,self.name)

class MatchTypeError(Exception):
    def __init__(self,kind,rule):
        self.kind = kind
        self.rule = rule
    def __str__(self):
        return "规则 %s 对应的种类 %s 不受支持"%(self.rule,self.kind)

class AdminRunError(Exception):
    def __init__(self,api):
        self.api  = api
    def __str__(self):
        return "重载API映射 %s 出现错误\n可能原因:\n1.未注册映射\n2.重载映射出错"%self.api

class AdminStyleError(Exception):
    def __init__(self,api):
        self.api  = api
    def __str__(self):
        return "API %s 映射的表与实际数据格式相冲突"%self.api


from .error import CSVkitImportError
from time import time
try:
    from prettytable import PrettyTable
except:
    raise CSVkitImportError("prettytable")





class CSVobjects(object):
    def __init__(self,_content:list,_fileMessage:dict):
        """
        :param _content: The key Value-List of `func-op`
        """
        self._content = _content
        self._fileMessage = _fileMessage
        # 实际继承csv.reader()类
    @property
    def size(self):
        """
        :return: Byte

        * 1 MB = 1024 KB = 1048576 Bytes
        """
        return "%s Byte"%self._fileMessage["size"]


    @property
    def modify_time(self):
        return self._fileMessage["mtime"]

    @property
    def create_time(self):
        return self._fileMessage["ctime"]

    @property
    def rows(self) -> int or str:
        try:
            return len(self._content)
        except Exception as e:
            return "error:%s"%str(e)

    @property
    def columns(self) -> int or str:
        try:
            max = len(self._content[0])
            for each_row in self._content[1:]:
                if max < len(each_row):
                    max = len(each_row)
                else:
                    pass
            return max
        except IndexError:
            return "error:First row have no datas"

    @staticmethod
    def __get_iterator(start_value:int,end_value:int,equal:bool):
        if equal == True:
            while start_value <= end_value:
                yield start_value
                start_value += 1
        else:
            while start_value < end_value:
                yield start_value
                start_value += 1

    def specifyROW(self,**kwargs) -> str or list:
        """
        :param kwargs: start_num:int,end_num:int,equal:bool
        :return: List

        * When equal is true, data in the range `[start_num,end_num]` will be returned

        * When equal is False, data in the range `[start_num,end_num)` will be returned
        """
        if "start_num" not in kwargs:
            kwargs.setdefault("start_num",0)
        if "end_num" not in kwargs:
            kwargs.setdefault("end_num",len(self._content))
        if "equal" not in kwargs:
            kwargs.setdefault("equal",False)

        start_num = int(kwargs["start_num"])
        end_num = int(kwargs["end_num"])

        if start_num >= end_num:
            return "start_num '%s' must smaller than '%s" % (start_num, end_num)
        elif start_num < 0 or end_num < 0:
            return "input must bigger than 0 or it is 0."
        else:
            return_list:[] = []
            for i in self.__get_iterator(start_num,end_num,kwargs["equal"]):
                return_list.append(self._content[i])
            return return_list

    @property
    def preview(self):
        """
        :return: prettytable.prettytable.PrettyTable
        """
        start = time()
        header_length = len(self._content[0])
        # Calculate maximum width
        max = header_length
        for each_row in self._content[1:]:
            if max < len(each_row):
                max = len(each_row)
            else:
                pass
        # Setting the header
        Complete_header_list = self._content[0][:]
        if header_length == max:
            table = PrettyTable(Complete_header_list)
        else:
            Gaps = max - header_length
            for _ in self.__get_iterator(0, Gaps, False):
                Complete_header_list.append("")
            table = PrettyTable(Complete_header_list)
        # Completing every list but first header
        if len(self._content) > 1000:
            pre_num = 1000
        else:
            pre_num = len(self._content)
        for each_row_last in self._content[1:pre_num]:
            ready_to_add_row = each_row_last[:]
            if len(ready_to_add_row) < max:
                Gaps = max - len(ready_to_add_row)
                for _ in self.__get_iterator(0,Gaps,False):
                    ready_to_add_row.append("")
            else:
                pass
            table.add_row(ready_to_add_row)
        elapsed = (time() - start)
        print("Process ends after %s seconds,Range is the first %s rows"%(elapsed,pre_num))

        return table


class WriteCSVobject(CSVobjects):
    def __init__(self,_content:list,_fileMessage:dict,_path:str):
        CSVobjects.__init__(self,_content,_fileMessage)
        self._path = _path

    @property
    def path(self):
        return self._path






import random
from typing import Dict,Union,Optional
USER_AGENT_LIST = [
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/22.0.1207.1 Safari/537.1",
    "Mozilla/5.0 (X11; CrOS i686 2268.111.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.6 (KHTML, like Gecko) Chrome/20.0.1092.0 Safari/536.6",
    "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.6 (KHTML, like Gecko) Chrome/20.0.1090.0 Safari/536.6",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/19.77.34.5 Safari/537.1",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.9 Safari/536.5",
    "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.36 Safari/536.5",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1063.0 Safari/536.3",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1063.0 Safari/536.3",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1063.0 Safari/536.3",
    "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1062.0 Safari/536.3",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1062.0 Safari/536.3",
    "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1061.1 Safari/536.3",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1061.1 Safari/536.3",
    "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1061.1 Safari/536.3",
    "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1061.0 Safari/536.3",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.24 (KHTML, like Gecko) Chrome/19.0.1055.1 Safari/535.24",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.24 (KHTML, like Gecko) Chrome/19.0.1055.1 Safari/535.24"
]

def returnUA(cookie:Optional[Union[str,bool]]) -> Dict[str,Union[str,int,float,bytes]]:
    num_random = random.randint(0, len(USER_AGENT_LIST) - 1)
    headers = {
        "User-Agent":USER_AGENT_LIST[num_random],
        "Cookie":cookie
    }
    return headers







from .operate import QSobject
from .error import ApiTypeError
from .models import DefaultSetting
from .struct import AnswerObject


def search(url:str,**kwargs) -> AnswerObject:
    """

    :param url: Str
    :param kwargs: API:Str cookie:Str
    :return: PrettyTable Object

    * The search() method contains one required parameter url, optional parameters API (the source of the search API), cookie (the main credentials for logging in, default cookie
      in `models.DefaultSetting().setting["default_cookie_Chaoxing"])`


    """
    # Make default settings
    if 'API' not in kwargs:
        kwargs.setdefault("API",DefaultSetting().default_api)
    if 'cookie' not in kwargs:
        kwargs.setdefault("cookie",DefaultSetting().default_cookie_Chaoxing)
    # Verify that the API interface is supported
    if kwargs["API"] not in DefaultSetting().api_support:
        raise ApiTypeError(kwargs["icodef"])
    print("You are using AnswerSearchkit {}, please wait for the program to finish...".format("1.0.0 Beta"))
    return QSobject().search_question(url,**kwargs)


import setuptools
from setuptools import find_packages

setuptools.setup(
    name = "PyUniversalKit",
    version = "1.0.6",
    author = "MatthewYt",
    author_email = "2158707744@qq.com",
    description = "Hello,PyUniversalKit!",
    packages = find_packages(),
    url='https://github.com/yangtang-special/PyUniversalKit',
    long_description_content_type = "text/markdown",
    data_files = ["README.md"],
    long_description = open('README.md',encoding="utf-8").read(),

)




from .error import ThirdPackageImportError,AdminStyleError,AdminRunError
from .admin import mapping
from PyUniversalKit import CSVkit
import sys

try:
    from prettytable import PrettyTable
except:
    raise ThirdPackageImportError("prettytable")

class AnswerObject():
    def __init__(self,_content,_kind):
        self._content = _content
        self._kind = _kind
    @staticmethod
    def __iterator(start_num, end_num):
        while start_num < end_num:
            yield start_num
            start_num += 1

    def __return_table(self) -> PrettyTable:
        try:
            mapping_reload_flag = mapping[self._kind]
        except:
            raise AdminRunError(self._kind)

        print("\nAPI %s mapping reload complete.\n" % self._kind)

        if mapping_reload_flag == True:
            try:
                PATH = "%s.csv" % self._kind
                header = CSVkit.read(PATH).specifyROW()
                table = PrettyTable(header[0], encoding=sys.stdout.encoding)
                num = 1
                for i in self._content:
                    table.add_row(["%s." % num, i[0], i[1], i[2],self._kind])
                    num += 1

                return table
            except:
                raise AdminStyleError(self._kind)



    @property
    def preview(self) -> PrettyTable:
        return self.__return_table()

    @property
    def original_data(self) -> list:
        return self._content

